<?php

namespace Drupal\color_picker_field\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;

/**
 * Plugin implementation of the 'color_picker' field type.
 *
 * @FieldType(
 *   id = "color_picker",
 *   label = @Translation("Color Picker"),
 *   description = @Translation("A field for setting bgcolor for body field."),
 *   default_widget = "color_picker_widget",
 *   default_formatter = "color_picker_formatter"
 * )
 */
class ColorPickerItem extends FieldItemBase {

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    return [
      'columns' => [
        'value' => [
          'type' => 'varchar',
          'length' => 7,
          'not null' => FALSE,
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    return empty($this->get('value')->getValue());
  }

  /**
   * {@inheritdoc}
   */
  public static function createFieldDefinition(FieldStorageDefinitionInterface $field_definition) {
    $definition = parent::createFieldDefinition($field_definition);
    $definition->setSetting('default_value', '#ffffff');
    return $definition;
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['value'] = DataDefinition::create('string')
      ->setLabel(t('Color Value'))
      ->setRequired(TRUE);

    return $properties;
  }

}
