<?php

namespace Drupal\color_picker_field\Plugin\Field\FieldWidget;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'color_picker' widget.
 *
 * @FieldWidget(
 *   id = "color_picker_widget",
 *   label = @Translation("Color Picker Widget"),
 *   field_types = {
 *     "color_picker"
 *   }
 * )
 */
class ColorPickerWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'color_picker_format' => '#HEX',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element['#type'] = 'color';
    $element['#title'] = $this->t('Select Text color');

    $element['#default_value'] = isset($items[$delta]) ? $items[$delta]->value : '#ffffff';

    $element['#attributes']['class'][] = 'color-picker-field';

    $element['#attached']['library'][] = 'color_picker_field/background_color';

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function extractFormValues(FieldItemListInterface $items, array $form, FormStateInterface $form_state) {
    // The key where the color value will be stored.
    $field_name = $this->fieldDefinition->getName();

    $path = array_merge($form['#parents'], [$field_name]);
    $key_exists = NULL;
    $values = NestedArray::getValue($form_state->getValues(), $path, $key_exists);

    if ($key_exists) {
      if (is_string($values)) {
        $values = [$values];
      }

      if (!$this->handlesMultipleValues()) {
        unset($values['add_more']);

        foreach ($values as $delta => &$value) {
          if (is_string($value)) {
            $value = ['value' => $value];
          }
          $value['_original_delta'] = $delta;
        }

        usort($values, function ($a, $b) {
          return SortArray::sortByKeyInt($a, $b, '_weight');
        });
      }

      $values = $this->massageFormValues($values, $form, $form_state);

      $items->setValue($values);
      $items->filterEmptyItems();

      $field_state = static::getWidgetState($form['#parents'], $field_name, $form_state);
      foreach ($items as $delta => $item) {
        $field_state['original_deltas'][$delta] = $item->_original_delta ?? $delta;
        unset($item->_original_delta, $item->_weight, $item->_actions);
      }
      static::setWidgetState($form['#parents'], $field_name, $form_state, $field_state);
    }
  }

  /**
   * Get the current value of the color field.
   */
  protected function getValue(FieldItemListInterface $items, $delta) {
    return isset($items[$delta]) && isset($items[$delta]->value) ? $items[$delta]->value : '#ffffff';
  }

}
